@@ -1,6 +1,10 @@
 from __future__ import print_function, unicode_literals
 
-import sys, collections, types, re, functools
+import sys
+import collections
+import types
+import re
+import functools
 from unittest.case import TestCase
 from collections import namedtuple, Mapping, Iterable
 
@@ -20,13 +24,16 @@ _repr = Repr().repr
 
 unittest_case = TestCase(methodName='__init__')
 
+
 def _format(message, *args):
     return message.format(*map(_repr, args))
 
+
 class EnsureError(AssertionError):
     # TODO: preserve original error type, define API for introspecting EnsureError
     pass
 
+
 class Inspector(object):
     def __init__(self, subject=None, error_factory=EnsureError, catch=Exception):
         self._orig_subject = subject
@@ -68,6 +75,7 @@ class Inspector(object):
             new_err = self._error_factory(err)
             raise new_err
 
+
 class IterableInspector(Inspector):
     def of(self, prototype):
         for item in self._subject:
@@ -81,6 +89,7 @@ class IterableInspector(Inspector):
                 raise self._error_factory(_format("Expected {} to be non-empty", item))
         return self.of(prototype)
 
+
 class MappingInspector(Inspector):
     def to(self, prototype):
         for value in self._subject.values():
@@ -92,16 +101,19 @@ class MappingInspector(Inspector):
                 raise self._error_factory(_format("Expected {} to be non-empty", value))
         return self.to(prototype)
 
+
 class AttributeInspector(Inspector):
     @property
     def which(self):
         return Ensure(self._subject)
 
+
 class KeyInspector(Inspector):
     @property
     def whose_value(self):
         return Ensure(self._subject)
 
+
 class CallableInspector(Inspector):
     def returns(self, value):
         """
@@ -112,6 +124,7 @@ class CallableInspector(Inspector):
     def __getattr__(self, item):
         return getattr(self._subject, item)
 
+
 class MultiInspector(Inspector):
     """
     Calls a list of inspector objects on a single subject.
@@ -127,10 +140,11 @@ class MultiInspector(Inspector):
                 inspector = self._get_inspector(subject)
                 inspect_method = getattr(inspector, item)
                 sub_inspectors.append(inspect_method(*args, **kwargs))
-            if not all(i == None for i in sub_inspectors):
+            if not all(i is None for i in sub_inspectors):
                 return MultiInspector(sub_inspectors)
         return inspect
 
+
 class MultiEnsure(MultiInspector):
     """
     Maps a single Ensure object to an iterable of subjects.
@@ -143,6 +157,7 @@ class MultiEnsure(MultiInspector):
     def _get_inspector(self, subject):
         return self._inspector(subject)
 
+
 class Ensure(Inspector):
     ''' Constructs a root-level inspector, which can perform a variety of checks (*predicates*) on subjects passed to
     it. If the checks do not pass, by default :class:`EnsureError` is raised. This can be configured by passing the
@@ -550,6 +565,7 @@ class Ensure(Inspector):
     #    import jsonschema
     #    self._run(jsonschema.validate, (self._subject, schema))
 
+
 class Check(object):
     """
     Like Ensure, but if a check fails, saves the error instead of raising it immediately. The error can then be acted
@@ -604,6 +620,7 @@ class Check(object):
         if self._exception:
             _callable(*args, **kwargs)
 
+
 def ensure_annotations(f):
     """
     Decorator to be used on functions with annotations. Runs type checks to enforce annotations. Raises
@@ -626,26 +643,33 @@ def ensure_annotations(f):
 
         >>> ensure.EnsureError: Argument y to <function f at 0x109b7c710> does not match annotation type <class 'float'>
     """
+    if f.__defaults__:
+        for rpos, value in enumerate(f.__defaults__):
+            if value is not None:
+                pos = f.__code__.co_argcount - len(f.__defaults__) + rpos
+                arg = f.__code__.co_varnames[pos]
+                if arg in f.__annotations__:
+                    templ = f.__annotations__[arg]
+                    if not isinstance(value, templ):
+                        msg = "Default argument {arg} to {f} does not match annotation type {t}"
+                        raise EnsureError(msg.format(arg=arg, f=f, t=templ))
+    arg_properties = []
+    for pos, arg in enumerate(f.__code__.co_varnames):
+        if arg in f.__annotations__:
+            templ = f.__annotations__[arg]
+            arg_properties.append((arg, templ, pos))
     from functools import wraps
+
     @wraps(f)
     def wrapper(*args, **kwargs):
-        arg_pos = None
-        for arg, templ in f.__annotations__.items():
-            if arg == 'return':
-                continue
+        for arg, templ, pos in arg_properties:
+            if len(args) > pos:
+                value = args[pos]
             elif arg in kwargs:
                 value = kwargs[arg]
             else:
-                if arg_pos is None:
-                    arg_pos = {arg: pos for pos, arg in enumerate(f.__code__.co_varnames)}
-                if arg in arg_pos:
-                    pos = arg_pos[arg]
-                    if len(args) > pos:
-                        value = args[pos]
-                    else:
-                        continue
-                else:
-                    continue
+                continue
+
             if not isinstance(value, templ):
                 msg = "Argument {arg} to {f} does not match annotation type {t}"
                 raise EnsureError(msg.format(arg=arg, f=f, t=templ))
